// Задача заключается в раскраске рёбер двудольного графа в наименьшее количество цветов:
// преподаватели и группы -- это вершины, а занятия -- это рёбра. Цвет означает порядок занятия в
// расписании. На этот счёт можно ознакомиться с теорией:
// 1. https://neerc.ifmo.ru/wiki/index.php?title=Рёберная_раскраска_двудольного_графа
// 2. http://www.cs.colorado.edu/department/publications/reports/docs/CU-CS-123-78.pdf
// 3. Поиск максимального паросочетания: http://e-maxx.ru/algo/kuhn_matching

#include <algorithm>
#include <cassert>
#include <cstdio>
#include <cstdlib>
#include <cstring>
#include <iostream>
#include <vector>

using namespace std;

const bool DEBUG = false;
const int maxn = 100;
const int maxm = 100;
const int maxk = 1000;

// Граф. Хранится в виде списков смежности для левой вершины. Булево значение указывает, было ли это
// ребро уже использовано при построении какого-то из паросочетаний.
vector<pair<int, bool>> g[maxn];
// Реально ли это ребро.
vector<bool> real[maxn];

// Посещали ли мы эту вершину в текущем поиске в глубину. Чтобы не обнулять массив постоянно, мы
// храним номер последнего поиска в глубину, в котором мы посещали эту вершину.
int vis[maxn];
int dfs_round = 0;

// Текущее паросочетание. Хранятся пары для вершин из правой доли и наш номер в списке смежности
// нашей пары.
int match[maxm], match_index[maxm];

// Ответ в виде списка занятий для каждого пункта в расписании.
vector<vector<pair<int, int>>> ans;

// Построение чередующейся цепочки поиском в глубину из левой доли. Здесь v --
// номер вершины в левой доле. Функция возращает, удалось ли построить цепочку,
// и сама подправляет паросочетание в случае успеха.
bool dfs(int v) {
  if (vis[v] == dfs_round) return false;
  vis[v] = dfs_round;
  for (int i = 0; i < g[v].size(); i++) {
    if (!g[v][i].second) {
      int u = g[v][i].first;
      if (match[u] < 0 || dfs(match[u])) {
        // Подправляем паросочетание.
        match[u] = v;
        match_index[u] = i;
        return true;
      }
    }
  }
  return false;
}

int main() {
  if (!DEBUG) {
    freopen("input.txt", "r", stdin);
    freopen("output.txt", "w", stdout);
  }

  int n, m, k;
  cin >> n >> m >> k;
  assert(1 <= n && n <= maxn);
  assert(1 <= m && m <= maxm);
  assert(1 <= k && k <= maxk);
  for (int i = 0; i < k; i++) {
    int u, v;
    cin >> u >> v;
    u--; v--;
    assert(0 <= u && u < n);
    assert(0 <= v && v < m);
    g[u].push_back(make_pair(v, false));
    real[u].push_back(true);
  }

  // Дополняем наш граф до регулярного. Делаем так чтобы в долях было одинаковое число вершин, и
  // все имели одинаковую степень. Тогда каждый раз мы будем находить совершенное паросочетание,
  // и каждый раз после его удаления наш граф будет оставаться регулярным.
  n = m = max(n, m);
  int max_deg = 0;
  vector<int> degL(n), degR(m);
  for (int i = 0; i < n; i++) {
    degL[i] += g[i].size();
    for (int j = 0; j < g[i].size(); j++) {
      degR[g[i][j].first]++;
    }
  }
  for (int i = 0; i < n; i++) {
    max_deg = max(max_deg, degL[i]);
    max_deg = max(max_deg, degR[i]);
  }
  while (1) {
    bool added = false;
    for (int i = 0; i < n; i++) {
       if (degL[i] == max_deg) continue;
       for (int j = 0; j < n; j++) {
         if (degR[j] == max_deg) continue;
         added = true;
         degL[i]++;
         degR[j]++;
         g[i].push_back(make_pair(j, false));
         real[i].push_back(false);
         if (degL[i] == max_deg) break;
       }
    }
    if (!added) break;
  }
  for (int i = 0; i < n; i++) {
    assert(degL[i] == max_deg);
    assert(degR[i] == max_deg);
  }

  int took = 0;  // Сколько рёбер мы уже включили в паросочетания.
  while (took < max_deg * n) {
    memset(match, 0xff, sizeof(match));

    // Ищем максимальное паросочетание среди оставшихся ребёр методом чередующихся цепочек
    // (алгоритм Куна). Пытаемся строить чередующиеся цепочки, начиная с левой доли.
    for (int i = 0; i < n; i++) {
       dfs_round++;
       if (dfs(i)) {
         took++;
       }
    }

    // Добавляем найденное паросочетание в ответ и помечаем использованные рёбра.
    ans.emplace_back();
    for (int i = 0; i < m; i++) {
      if (match[i] >= 0) {
        if (real[match[i]][match_index[i]]) {
          ans.back().push_back(make_pair(match[i], i));
        }
        g[match[i]][match_index[i]].second = true;
      }
    }
  }

  cout << ans.size() << endl << endl;
  for (const auto &list : ans) {
    cout << list.size() << endl;
    for (const auto &c : list) {
      cout << c.first + 1 << " " << c.second + 1 << endl;
    }
    cout << endl;
  }

  if (!DEBUG) {
    fclose(stdin); fclose(stdout);
  }
  return 0;
}
